/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.operator.keycloak.user;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import java.util.Optional;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;

import de.ozgcloud.operator.Config;
import de.ozgcloud.operator.keycloak.OzgCloudCustomResourceStatus;
import de.ozgcloud.operator.keycloak.realm.OzgCloudKeycloakRealmTestFactory;
import de.ozgcloud.operator.keycloak.user.OzgCloudKeycloakUserSpec.KeycloakUserSpecUser;
import io.javaoperatorsdk.operator.api.reconciler.DeleteControl;

class KeycloakUserReconcilerTest {

	private final String REALM = OzgCloudKeycloakUserTestFactory.METADATA_NAMESPACE;

	@Spy
	@InjectMocks
	private KeycloakUserReconciler reconciler;

	@Mock
	private KeycloakUserService service;

	@Mock
	private KeycloakUserPreconditionService preconditionService;

	@Nested
	class TestAddUser {

		@BeforeEach
		void init() {
			when(preconditionService.getPreconditionErrors(any())).thenReturn(Optional.empty());
		}

		@Test
		void shouldCallServiceAddUser() {
			OzgCloudKeycloakUser user = OzgCloudKeycloakUserTestFactory.create();

			reconciler.reconcile(user, null);

			verify(service).createOrUpdateUser(user.getSpec(), OzgCloudKeycloakUserTestFactory.METADATA_NAMESPACE);
		}

		@Test
		void shouldUpdateStatus() {
			OzgCloudKeycloakUser user = OzgCloudKeycloakUserTestFactory.create();

			reconciler.reconcile(user, null);

			assertThat(user.getStatus().getStatus()).isEqualTo(OzgCloudCustomResourceStatus.OK);
		}

		@Nested
		class TestPreconditionError {

			private static final String ERROR_MESSAGE = "ErrorMessage";

			@Test
			void shouldSetPreconcitionErrorMessage() {
				when(preconditionService.getPreconditionErrors(any())).thenReturn(Optional.of(ERROR_MESSAGE));
				OzgCloudKeycloakUser user = OzgCloudKeycloakUserTestFactory.create();

				reconciler.reconcile(user, null);

				assertThat(user.getStatus().getMessage()).isEqualTo(ERROR_MESSAGE);
			}
		}
	}

	@DisplayName("Reconciler Cleanup")
	@Nested
	class TestReconcilerCleanup {

		@DisplayName("with keep_after_delete")
		@Nested
		class TestWithKeepAfterDelete {

			private final KeycloakUserSpecUser keycloakUser = KeycloakUserSpecUserTestFactory.createBuiler().build();
			private final OzgCloudKeycloakUserSpec userSpec = OzgCloudKeycloakUserSpecTestFactory.createBuilder().keepAfterDelete(true)
					.keycloakUser(keycloakUser).build();
			private final OzgCloudKeycloakUser user = OzgCloudKeycloakUserTestFactory.create(userSpec);

			@Test
			void shouldNotCallService() {
				reconciler.cleanup(user, null);

				verify(service, never()).deleteUser(userSpec, OzgCloudKeycloakUserTestFactory.METADATA_NAMESPACE);
			}

			@Test
			void shouldReturnDeleteControl() {
				var control = reconciler.cleanup(user, null);

				assertThat(control).usingRecursiveComparison().isEqualTo(DeleteControl.defaultDelete());
			}
		}

		@DisplayName("test cleanup")
		@Nested
		class TestCleanup {

			private final OzgCloudKeycloakUser user = OzgCloudKeycloakUserTestFactory.create();

			@Test
			void shouldCallDeleteUser() {
				when(preconditionService.realmExists(REALM)).thenReturn(true);

				reconciler.cleanup(user, null);

				verify(reconciler).deleteUser(user);
			}

			@Test
			void shouldReturnValueFromDeleteUser() {
				DeleteControl expected = DeleteControl.defaultDelete();
				when(reconciler.deleteUser(user)).thenReturn(expected);
				when(preconditionService.realmExists(REALM)).thenReturn(true);

				DeleteControl response = reconciler.cleanup(user, null);

				assertThat(response).isEqualTo(expected);
			}

			@Test
			void shouldCallRealmExists() {
				reconciler.cleanup(user, null);

				verify(preconditionService).realmExists(REALM);
			}

			@Test
			void shouldDeleteUserIfRealmExists() {
				when(preconditionService.realmExists(REALM)).thenReturn(true);

				reconciler.cleanup(user, null);

				verify(reconciler).deleteUser(user);
			}

			@Test
			void shouldNotDeleteUserIfRealmNotExists() {
				when(preconditionService.realmExists(REALM)).thenReturn(false);

				reconciler.cleanup(user, null);

				verify(reconciler, never()).deleteUser(user);
			}
		}

		@DisplayName("test delete")
		@Nested
		class TestDeleteUser {

			private final OzgCloudKeycloakUser user = OzgCloudKeycloakUserTestFactory.create();

			@Test
			void shouldCallServiceDelete() {
				reconciler.deleteUser(user);

				verify(service).deleteUser(user.getSpec(), OzgCloudKeycloakUserTestFactory.METADATA_NAMESPACE);
			}

			@Test
			void shouldReturnDeleteControl() {
				var control = reconciler.deleteUser(user);

				assertThat(control).usingRecursiveComparison().isEqualTo(DeleteControl.defaultDelete());
			}

			@Test
			void shouldRescheduleOnError() {
				when(preconditionService.realmExists(REALM)).thenReturn(true);
				doThrow(RuntimeException.class).when(service)
						.deleteUser(user.getSpec(), OzgCloudKeycloakRealmTestFactory.METADATA_NAMESPACE);

				var control = reconciler.cleanup(user, null);

				assertThat(control).usingRecursiveComparison()
						.isEqualTo(DeleteControl.noFinalizerRemoval().rescheduleAfter(Config.RECONCILER_RETRY_SECONDS_ON_ERROR));
			}

		}
	}
}
