/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.operator.keycloak.user;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import java.util.Optional;

import org.apache.commons.lang3.StringUtils;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.keycloak.representations.idm.UserRepresentation;
import org.mockito.ArgumentCaptor;
import org.mockito.Captor;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;

import io.fabric8.kubernetes.api.model.Secret;
import io.fabric8.kubernetes.client.dsl.Resource;

class KeycloakUserServiceTest {

	private static final String TEST_NAMESPACE = "TestNamespace";
	private static final String TEST_USERID = "TestUserId";

	@Spy
	@InjectMocks
	private KeycloakUserService service;

	@Mock
	private KeycloakUserRemoteService remoteService;

	@Mock
	private UserSecretService userSecretService;

	@Mock
	private KeycloakUserMapper userMapper;

	@DisplayName("Create or Update")
	@Nested
	class TestCreateOrUpdate {

		@Mock
		private Resource<Secret> resource;
		@Captor
		private ArgumentCaptor<OzgCloudKeycloakUserSpec> ozgCloudKeycloakUserSpecCaptor;

		@Test
		void shouldCallCreateAndSetUserPasswordIfNotExists() {
			OzgCloudKeycloakUserSpec userSpec = OzgCloudKeycloakUserSpecTestFactory.create();

			service.createOrUpdateUser(userSpec, TEST_NAMESPACE);

			verify(service).createAndSetUserPasswordIfNotExists(userSpec, TEST_NAMESPACE);
		}

		@Test
		void shouldCallUserRemoteServiceGetUserByName() {
			OzgCloudKeycloakUserSpec userSpec = OzgCloudKeycloakUserSpecTestFactory.create();

			service.createOrUpdateUser(userSpec, TEST_NAMESPACE);

			verify(remoteService).getUserByName(KeycloakUserSpecUserTestFactory.USERNAME, TEST_NAMESPACE);
		}

		@Test
		void shouldCallCreateUserIfNotExists() {
			var userSpec = OzgCloudKeycloakUserSpecTestFactory.create();
			when(remoteService.getUserByName(KeycloakUserSpecUserTestFactory.USERNAME, TEST_NAMESPACE))
					.thenReturn(Optional.empty());

			service.createOrUpdateUser(userSpec, TEST_NAMESPACE);

			verify(service).createUser(userSpec, TEST_NAMESPACE);
		}

		@Test
		void shouldCallUpdateUserIfAlreadyExists() {
			var userSpec = OzgCloudKeycloakUserSpecTestFactory.create();
			var remoteUser = UserRepresentationTestFactory.create();
			when(remoteService.getUserByName(KeycloakUserSpecUserTestFactory.USERNAME, TEST_NAMESPACE))
					.thenReturn(Optional.of(remoteUser));

			service.createOrUpdateUser(userSpec, TEST_NAMESPACE);

			verify(service).updateUser(userSpec, TEST_NAMESPACE, remoteUser);
		}
	}

	@Nested
	class TestCreateAndSetUserPasswordIfNotExists {

		@Test
		void shouldCallUserHasNoPassword() {
			OzgCloudKeycloakUserSpec userSpec = OzgCloudKeycloakUserSpecTestFactory.create();

			service.createAndSetUserPasswordIfNotExists(userSpec, TEST_NAMESPACE);

			verify(service).userHasNoPassword(userSpec, TEST_NAMESPACE);
		}

		@DisplayName("on user has password")
		@Nested
		class TestOnUserHasPassword {

			@BeforeEach
			void mock() {
				doReturn(false).when(service).userHasNoPassword(any(), any());
			}

			@Test
			void shouldNotReadSecretFromCluster() {
				OzgCloudKeycloakUserSpec userSpec = OzgCloudKeycloakUserSpecTestFactory.create();

				service.createAndSetUserPasswordIfNotExists(userSpec, TEST_NAMESPACE);

				verify(userSecretService, never()).getOrCreateClusterSecret(userSpec, TEST_NAMESPACE);
			}

			@Test
			void shouldNotCallUserSecretServiceGetOrCreateClusterSecret() {
				OzgCloudKeycloakUserSpec userSpec = OzgCloudKeycloakUserSpecTestFactory.create();

				service.createAndSetUserPasswordIfNotExists(userSpec, TEST_NAMESPACE);

				verify(userSecretService, never()).getPasswordFromSecret(any(Secret.class));
			}
		}

		@DisplayName("user has no password")
		@Nested
		class TestOnUserHasNoPassword {

			@BeforeEach
			void mock() {
				doReturn(true).when(service).userHasNoPassword(any(), eq(TEST_NAMESPACE));
			}

			@Test
			void shouldGetOrCreateClusterSecret() {
				OzgCloudKeycloakUserSpec userSpec = OzgCloudKeycloakUserSpecTestFactory.create();

				service.createAndSetUserPasswordIfNotExists(userSpec, TEST_NAMESPACE);

				verify(userSecretService).getOrCreateClusterSecret(userSpec, TEST_NAMESPACE);
			}

			@Test
			void shouldUpdateUserPassword() {
				String NEWPASSWORD = "NewPassword";
				var userWithoutPassword = OzgCloudKeycloakUserSpecTestFactory.createBuilder()
						.keycloakUser(KeycloakUserSpecUserTestFactory.createBuiler().password(StringUtils.EMPTY).build()).build();
				when(userSecretService.getPasswordFromSecret(any())).thenReturn(NEWPASSWORD);

				service.createAndSetUserPasswordIfNotExists(userWithoutPassword, TEST_NAMESPACE);

				assertThat(userWithoutPassword.getKeycloakUser().getPassword()).isEqualTo(NEWPASSWORD);
			}
		}

	}

	@DisplayName("Test user has no password")
	@Nested
	class TestUserHasNoPassword {

		@Test
		void testUserHasNoPasswordTrue() {
			OzgCloudKeycloakUserSpec user = OzgCloudKeycloakUserSpecTestFactory.create();

			boolean userHasNoPassword = service.userHasNoPassword(user, TEST_NAMESPACE);

			assertThat(userHasNoPassword).isFalse();
		}

		@Test
		void testUserHasNoPasswordFalse() {
			OzgCloudKeycloakUserSpec user = createUserWithoutPassword();

			boolean userHasNoPassword = service.userHasNoPassword(user, TEST_NAMESPACE);

			assertThat(userHasNoPassword).isTrue();
		}

		private OzgCloudKeycloakUserSpec createUserWithoutPassword() {
			return OzgCloudKeycloakUserSpecTestFactory.createBuilder()
					.keycloakUser(KeycloakUserSpecUserTestFactory.createBuiler()
							.password(null).build())
					.build();
		}
	}

	@Nested
	class TestCreateUser {

		@Test
		void shouldCallRemoteServiceCreateUser() {
			var userSpec = OzgCloudKeycloakUserSpecTestFactory.create();
			UserRepresentation userRepresentation = UserRepresentationTestFactory.create();
			when(userMapper.map(userSpec)).thenReturn(userRepresentation);

			service.createUser(userSpec, TEST_NAMESPACE);

			verify(remoteService).createUser(userRepresentation, TEST_NAMESPACE);
		}
	}

	@Nested
	class TestUpdateUser {

		@Test
		void shouldUpdateUserIfEnabled() {
			var userSpec = OzgCloudKeycloakUserSpecTestFactory.create();
			UserRepresentation userRepresentation = UserRepresentationTestFactory.create();
			when(userMapper.update(userRepresentation, userSpec)).thenReturn(userRepresentation);

			service.updateUser(userSpec, TEST_NAMESPACE, userRepresentation);

			verify(remoteService).updateUser(any(UserRepresentation.class), eq(TEST_NAMESPACE));
		}

		@Test
		void shouldNotUpdateUserIfDisabled() {
			var userSpec = OzgCloudKeycloakUserSpecTestFactory.createBuilder()
					.updateUser(false).build();

			service.updateUser(userSpec, TEST_NAMESPACE, UserRepresentationTestFactory.create());

			verify(remoteService, never()).updateUser(null, TEST_NAMESPACE);
		}
	}

	@Nested
	class TestDeleteUser {

		@Test
		void shouldCallUserMapper() {

			var testUser = OzgCloudKeycloakUserSpecTestFactory.create();

			service.deleteUser(testUser, TEST_NAMESPACE);

			verify(userMapper).map(testUser);
		}

		@Test
		void shouldCallUserRemoteServiceGetUser() {

			var userRepresentation = UserRepresentationTestFactory.create();
			when(userMapper.map(any())).thenReturn(userRepresentation);

			service.deleteUser(OzgCloudKeycloakUserSpecTestFactory.create(), TEST_NAMESPACE);

			verify(remoteService).getUserByName(userRepresentation.getUsername(), TEST_NAMESPACE);
		}

		@Test
		void shouldCallUserRemoteServiceDeleteUser() {

			var userRepresentation = UserRepresentationTestFactory.create();
			when(userMapper.map(any())).thenReturn(userRepresentation);
			when(remoteService.getUserByName(UserRepresentationTestFactory.USERNAME, TEST_NAMESPACE)).thenReturn(Optional.of(userRepresentation));
			userRepresentation.setId(TEST_USERID);

			service.deleteUser(OzgCloudKeycloakUserSpecTestFactory.create(), TEST_NAMESPACE);

			verify(remoteService).deleteUser(TEST_USERID, TEST_NAMESPACE);
		}
	}
}
