package de.ozgcloud.operator.keycloak.user;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import java.util.Base64;

import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.RepeatedTest;
import org.junit.jupiter.api.Test;
import org.mockito.Spy;

import de.ozgcloud.operator.keycloak.user.OzgCloudKeycloakUserSpec.KeycloakUserSpecUser;

class UserSecretBuilderTest {

	private final static String NAME = "dummyName";
	private final static String NAMESPACE = "dummyNamespace";

	@Spy
	private UserSecretBuilder builder;

	@DisplayName("Build")
	@Nested
	class TestBuild {

		private final KeycloakUserSpecUser userSpec = KeycloakUserSpecUserTestFactory.create();

		@Test
		void shouldHaveType() {
			var secret = builder.build(NAME, userSpec, NAMESPACE);

			assertThat(secret.getType()).isEqualTo(UserSecretBuilder.SECRET_TYPE);
		}

		@Test
		void shouldHaveUserName() {
			var secret = builder.build(NAME, userSpec, NAMESPACE);

			assertThat(secret.getStringData()).containsEntry(UserSecretBuilder.SECRET_NAME_FIELD, KeycloakUserSpecUserTestFactory.USERNAME);
		}

		@Test
		void shouldCallGeneratePassword() {
			builder.build(NAME, userSpec, NAMESPACE);

			verify(builder).generatePassword();
		}

		@Test
		void shouldHavePassword() {
			doReturn(SecretTestFactory.PASSWORD).when(builder).generatePassword();

			var secret = builder.build(NAME, userSpec, NAMESPACE);

			assertThat(secret.getData()).containsEntry(UserSecretBuilder.SECRET_PASSWORD_FIELD, SecretTestFactory.PASSWORD);
		}

		@DisplayName("metadata")
		@Nested
		class TestMetaData {

			@Test
			void shouldHaveName() {
				var secret = builder.build(NAME, userSpec, NAMESPACE);

				assertThat(secret.getMetadata().getName()).isEqualTo(NAME);
			}

			@Test
			void shouldHaveNamespace() {
				var secret = builder.build(NAME, userSpec, NAMESPACE);

				assertThat(secret.getMetadata().getNamespace()).isEqualTo(NAMESPACE);
			}
		}

		@DisplayName("generate password")
		@Nested
		class TestGeneratePassword {

			@RepeatedTest(100)
			void shouldHaveSize() {
				var password = decode(builder.generatePassword());

				assertThat(password).hasSize(24);
			}

			@RepeatedTest(100)
			void shouldHaveUpperCaseLetters() {
				var password = decode(builder.generatePassword());

				assertThat(password).matches(".*[A-Z].*");
			}

			@RepeatedTest(100)
			void shouldHaveLowercaseLetters() {
				var password = decode(builder.generatePassword());

				assertThat(password).matches(".*[a-z].*");
			}

			@RepeatedTest(100)
			void shouldContainNumerics() {
				var password = decode(builder.generatePassword());

				assertThat(password).matches(".*[0-9].*");
			}

			@RepeatedTest(100)
			void shouldEncode() {
				builder.generatePassword();

				verify(builder).encode(any());
			}

			private String decode(String strValue) {
				return new String(Base64.getDecoder().decode(strValue.getBytes()));
			}
		}
	}
}
